@@ -30,14 +30,14 @@ m.QueueManager = function(Config, queues)
 	this.accounts = {};
 
 	// the sorting is updated on priority change.
-	var self = this;
 	this.queueArrays = [];
 	for (var p in this.queues)
 	{
 		this.accounts[p] = new API3.Resources();
-		this.queueArrays.push([p,this.queues[p]]);
+		this.queueArrays.push([p, this.queues[p]]);
 	}
-	this.queueArrays.sort(function (a,b) { return (self.priorities[b[0]] - self.priorities[a[0]]) });
+	var priorities = this.priorities;
+	this.queueArrays.sort(function (a,b) { return (priorities[b[0]] - priorities[a[0]]); });
 };
 
 m.QueueManager.prototype.getAvailableResources = function(gameState)
@@ -48,30 +48,29 @@ m.QueueManager.prototype.getAvailableResources = function(gameState)
 	return resources;
 };
 
-m.QueueManager.prototype.getTotalAccountedResources = function(gameState)
+m.QueueManager.prototype.getTotalAccountedResources = function()
 {
 	var resources = new API3.Resources();
-	for (var key in this.queues)
+	for (let key in this.queues)
 		resources.add(this.accounts[key]);
 	return resources;
 };
 
 m.QueueManager.prototype.currentNeeds = function(gameState)
 {
-	var needed = new API3.Resources();
+	let needed = new API3.Resources();
 	//queueArrays because it's faster.
-	for (var i in this.queueArrays)
+	for (let q of this.queueArrays)
 	{
-		var name = this.queueArrays[i][0];
-		var queue = this.queueArrays[i][1];
-		if (queue.length() == 0 || !queue.queue[0].isGo(gameState))
+		let queue = q[1];
+		if (!queue.length() || !queue.queue[0].isGo(gameState))
 			continue;
 		var costs = queue.queue[0].getCost();
 		needed.add(costs);
 	}
 	// get out current resources, not removing accounts.
-	var current = gameState.getResources();
-	for (var res of needed.types)
+	let current = gameState.getResources();
+	for (let res of needed.types)
 		needed[res] = Math.max(0, needed[res] - current[res]);
 
 	return needed;
@@ -81,6 +80,11 @@ m.QueueManager.prototype.currentNeeds = function(gameState)
 // TODO: many things.
 m.QueueManager.prototype.wantedGatherRates = function(gameState)
 {
+	if (gameState.ai.playedTurn == 0)
+	{
+		// default values for first turn when we have not yet set our queues.
+		return { "food": 10, "wood": 10, "stone": 0, "metal": 0 };
+	}
 	// get out current resources, not removing accounts.
 	var current = gameState.getResources();
 	// short queue is the first item of a queue, assumed to be ready in 30s
@@ -91,17 +95,16 @@ m.QueueManager.prototype.wantedGatherRates = function(gameState)
 	var totalLong = { "food": 0, "wood": 0, "stone": 0, "metal": 0 };
 	var total;
 	//queueArrays because it's faster.
-	for (var i in this.queueArrays)
+	for (let q of this.queueArrays)
 	{
-		var name = this.queueArrays[i][0];
-		var queue = this.queueArrays[i][1];
+		var queue = q[1];
 		if (queue.paused)
 			continue;
-		for (var j = 0; j < queue.length(); ++j)
+		for (let j = 0; j < queue.length(); ++j)
 		{
 			if (j > 1)
 				break;
-			var cost = queue.queue[j].getCost();
+			let cost = queue.queue[j].getCost();
 			if (queue.queue[j].isGo(gameState))
 			{
 				if (j == 0)
@@ -111,7 +114,7 @@ m.QueueManager.prototype.wantedGatherRates = function(gameState)
 			}
 			else
 				total = totalLong;
-			for (var type in total)
+			for (let type in total)
 				total[type] += cost[type];
 			if (!queue.queue[j].isGo(gameState))
 				break;
@@ -182,8 +185,6 @@ m.QueueManager.prototype.printQueues = function(gameState)
 // nice readable HTML version.
 m.QueueManager.prototype.HTMLprintQueues = function(gameState)
 {
-	if (!m.DebugEnabled())
-		return;
 	var strToSend = [];
 	strToSend.push("<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01//EN\"> <html> <head> <title>Aegis Queue Manager</title> <link rel=\"stylesheet\" href=\"table.css\">  </head> <body> <table> <caption>Aegis Build Order</caption> ");
 	for (var i in this.queues)
@@ -241,6 +242,21 @@ m.QueueManager.prototype.clear = function()
 		this.queues[i].empty();
 };
 
+/**
+ * set accounts of queue i from the unaccounted resources
+ */
+m.QueueManager.prototype.setAccounts = function(gameState, cost, i)
+{
+	var available = this.getAvailableResources(gameState);
+	for (var res of this.accounts[i].types)
+	{
+		if (this.accounts[i][res] >= cost[res])
+			continue;
+		var diff = Math.min(available[res], cost[res] - this.accounts[i][res]);
+		this.accounts[i][res] += diff;
+	}
+};
+
 /**
  * transfer accounts from queue i to queue j
  */
@@ -387,13 +403,13 @@ m.QueueManager.prototype.switchResource = function(gameState, res)
 // Start the next item in the queue if we can afford it.
 m.QueueManager.prototype.startNextItems = function(gameState)
 {
-	for (var i in this.queueArrays)
+	for (let q of this.queueArrays)
 	{
-		var name = this.queueArrays[i][0];
-		var queue = this.queueArrays[i][1];
+		let name = q[0];
+		let queue = q[1];
 		if (queue.length() > 0 && !queue.paused)
 		{
-			var item = queue.getNext();
+			let item = queue.getNext();
 			if (this.accounts[name].canAfford(item.getCost()) && item.canStart(gameState))
 			{
 				this.finishingTime = gameState.ai.elapsedTime;
@@ -423,6 +439,9 @@ m.QueueManager.prototype.update = function(gameState)
 		this.priorities[i] = 1;  // TODO: make the Queue Manager not die when priorities are zero.
 	}
 
+	// Pause or unpause queues depending on the situation
+	this.checkPausedQueues(gameState);
+
 	// Let's assign resources to plans that need them
 	this.distributeResources(gameState);
 
@@ -435,6 +454,56 @@ m.QueueManager.prototype.update = function(gameState)
 	Engine.ProfileStop();
 };
 
+// Recovery system: if short of workers after an attack, pause (and reset) some queues to favor worker training
+m.QueueManager.prototype.checkPausedQueues = function(gameState)
+{
+	let numWorkers = 0;
+	gameState.getOwnUnits().forEach (function (ent) {
+		if (ent.getMetadata(PlayerID, "role") == "worker")
+			numWorkers++;
+	});
+	gameState.getOwnTrainingFacilities().forEach(function(ent) {
+		ent.trainingQueue().forEach(function(item) {
+			if (item.metadata && item.metadata.role && item.metadata.role == "worker")
+				numWorkers += item.count;
+		});
+	});
+
+	var workersMin = Math.min(Math.max(12, 24 * this.Config.popScaling), this.Config.Economy.popForTown);
+	for (let q in this.queues)
+	{
+		let toBePaused = false;
+		if (gameState.ai.HQ.numActiveBase() === 0)
+			toBePaused = (q !== "dock" && q !== "civilCentre");
+		else if (numWorkers < workersMin / 3)
+			toBePaused = (q !== "citizenSoldier" && q !== "villager" && q !== "emergency");
+		else if (numWorkers < workersMin * 2 / 3)
+			toBePaused = (q === "civilCentre" || q === "economicBuilding"
+				|| q === "militaryBuilding" || q === "defenseBuilding"
+				|| q === "majorTech" || q === "minorTech" || q.indexOf("plan_") !== -1);
+		else if (numWorkers < workersMin)
+			toBePaused = (q === "civilCentre" || q === "defenseBuilding"
+				|| q == "majorTech" || q.indexOf("_siege") != -1 || q.indexOf("_champ") != -1);
+
+		let queue = this.queues[q];
+		if (!queue.paused && toBePaused)
+		{
+			queue.paused = true;
+			this.accounts[q].reset();
+		}
+		else if (queue.paused && !toBePaused)
+			queue.paused = false;
+
+		// And reduce the batch sizes of attack queues
+		if (q.indexOf("plan_") != -1 && numWorkers < workersMin && queue.queue[0])
+		{
+			queue.queue[0].number = 1;
+			if (queue.queue[1])
+				queue.queue[1].number = 1;
+		}
+	}
+};
+
 m.QueueManager.prototype.pauseQueue = function(queue, scrapAccounts)
 {
 	if (this.queues[queue])
@@ -474,34 +543,34 @@ m.QueueManager.prototype.unpauseAll = function(but)
 
 m.QueueManager.prototype.addQueue = function(queueName, priority)
 {
-	if (this.queues[queueName] == undefined)
-	{
-		this.queues[queueName] = new m.Queue();
-		this.priorities[queueName] = priority;
-		this.accounts[queueName] = new API3.Resources();
+	if (this.queues[queueName] !== undefined)
+		return;
 
-		var self = this;
-		this.queueArrays = [];
-		for (var p in this.queues)
-			this.queueArrays.push([p,this.queues[p]]);
-		this.queueArrays.sort(function (a,b) { return (self.priorities[b[0]] - self.priorities[a[0]]) });
-	}
+	this.queues[queueName] = new m.Queue();
+	this.priorities[queueName] = priority;
+	this.accounts[queueName] = new API3.Resources();
+
+	this.queueArrays = [];
+	for (var p in this.queues)
+		this.queueArrays.push([p, this.queues[p]]);
+	var priorities = this.priorities;
+	this.queueArrays.sort(function (a,b) { return (priorities[b[0]] - priorities[a[0]]); });
 };
 
 m.QueueManager.prototype.removeQueue = function(queueName)
 {
-	if (this.queues[queueName] !== undefined)
-	{
-		delete this.queues[queueName];
-		delete this.priorities[queueName];
-		delete this.accounts[queueName];
+	if (this.queues[queueName] === undefined)
+		return;
+
+	delete this.queues[queueName];
+	delete this.priorities[queueName];
+	delete this.accounts[queueName];
 		
-		var self = this;
-		this.queueArrays = [];
-		for (var p in this.queues)
-			this.queueArrays.push([p,this.queues[p]]);
-		this.queueArrays.sort(function (a,b) { return (self.priorities[b[0]] - self.priorities[a[0]]) });
-	}
+	this.queueArrays = [];
+	for (var p in this.queues)
+		this.queueArrays.push([p, this.queues[p]]);
+	var priorities = this.priorities;
+	this.queueArrays.sort(function (a,b) { return (priorities[b[0]] - priorities[a[0]]); });
 };
 
 m.QueueManager.prototype.getPriority = function(queueName)
@@ -513,13 +582,49 @@ m.QueueManager.prototype.changePriority = function(queueName, newPriority)
 {
 	if (this.Config.debug > 1)
 		API3.warn(">>> Priority of queue " + queueName + " changed from " + this.priorities[queueName] + " to " + newPriority);
-	var self = this;
 	if (this.queues[queueName] !== undefined)
 		this.priorities[queueName] = newPriority;
+	var priorities = this.priorities;
+	this.queueArrays.sort(function (a,b) { return (priorities[b[0]] - priorities[a[0]]); });
+};
+
+m.QueueManager.prototype.Serialize = function()
+{
+	var accounts = {};
+	var queues = {};
+	for (let p in this.queues)
+	{
+		queues[p] = this.queues[p].Serialize();
+		accounts[p] = this.accounts[p].Serialize();
+		if (this.Config.debug == -100)
+			API3.warn("queueManager serialization: queue " + p + " >>> " + uneval(queues[p])
+				+ " with accounts " + uneval(accounts[p]));
+	}
+
+	return {
+		"priorities": this.priorities,
+		"queues": queues,
+		"accounts": accounts
+	};
+};
+
+m.QueueManager.prototype.Deserialize = function(gameState, data)
+{
+	this.priorities = data.priorities;
+	this.queues = {};
+	this.accounts = {};
+
+	// the sorting is updated on priority change.
 	this.queueArrays = [];
-	for (var p in this.queues)
-		this.queueArrays.push([p,this.queues[p]]);
-	this.queueArrays.sort(function (a,b) { return (self.priorities[b[0]] - self.priorities[a[0]]) });
+	for (let p in data.queues)
+	{
+		this.queues[p] = new m.Queue();
+		this.queues[p].Deserialize(gameState, data.queues[p]);
+		this.accounts[p] = new API3.Resources();
+		this.accounts[p].Deserialize(data.accounts[p]);
+		this.queueArrays.push([p, this.queues[p]]);
+	}
+	this.queueArrays.sort(function (a,b) { return (data.priorities[b[0]] - data.priorities[a[0]]); });
 };
 
 return m;
