@@ -1,6 +1,6 @@
 function Attack() {}
 
-var bonusesSchema = 
+Attack.prototype.bonusesSchema = 
 	"<optional>" +
 		"<element name='Bonuses'>" +
 			"<zeroOrMore>" +
@@ -18,7 +18,7 @@ var bonusesSchema =
 		"</element>" +
 	"</optional>";
 
-var preferredClassesSchema =
+Attack.prototype.preferredClassesSchema =
 	"<optional>" +
 		"<element name='PreferredClasses' a:help='Space delimited list of classes preferred for attacking. If an entity has any of theses classes, it is preferred. The classes are in decending order of preference'>" +
 			"<attribute name='datatype'>" +
@@ -28,7 +28,7 @@ var preferredClassesSchema =
 		"</element>" +
 	"</optional>";
 
-var restrictedClassesSchema =
+Attack.prototype.restrictedClassesSchema =
 	"<optional>" +
 		"<element name='RestrictedClasses' a:help='Space delimited list of classes that cannot be attacked by this entity. If target entity has any of these classes, it cannot be attacked'>" +
 			"<attribute name='datatype'>" +
@@ -112,9 +112,9 @@ Attack.prototype.Schema =
 				"<element name='RepeatTime' a:help='Time between attacks (in milliseconds). The attack animation will be stretched to match this time'>" + // TODO: it shouldn't be stretched
 					"<data type='positiveInteger'/>" +
 				"</element>" +
-				bonusesSchema +
-				preferredClassesSchema +
-				restrictedClassesSchema +
+				Attack.prototype.bonusesSchema +
+				Attack.prototype.preferredClassesSchema +
+				Attack.prototype.restrictedClassesSchema +
 			"</interleave>" +
 		"</element>" +
 	"</optional>" +
@@ -139,9 +139,9 @@ Attack.prototype.Schema =
 					"<ref name='nonNegativeDecimal'/>" +
 				"</element>" +
 				"<element name='Spread' a:help='Radius over which missiles will tend to land (when shooting to the maximum range). Roughly 2/3 will land inside this radius (in metres). Spread is linearly diminished as the target gets closer.'><ref name='nonNegativeDecimal'/></element>" +
-				bonusesSchema +
-				preferredClassesSchema +
-				restrictedClassesSchema +
+				Attack.prototype.bonusesSchema +
+				Attack.prototype.preferredClassesSchema +
+				Attack.prototype.restrictedClassesSchema +
 				"<optional>" +
 					"<element name='Splash'>" +
 						"<interleave>" +
@@ -151,13 +151,27 @@ Attack.prototype.Schema =
 							"<element name='Hack' a:help='Hack damage strength'><ref name='nonNegativeDecimal'/></element>" +
 							"<element name='Pierce' a:help='Pierce damage strength'><ref name='nonNegativeDecimal'/></element>" +
 							"<element name='Crush' a:help='Crush damage strength'><ref name='nonNegativeDecimal'/></element>" +
-							bonusesSchema +
+							Attack.prototype.bonusesSchema +
 						"</interleave>" +
 					"</element>" +
 				"</optional>" +
 			"</interleave>" +
 		"</element>" +
 	"</optional>" +
+	"<optional>" +
+		"<element name='Capture'>" +
+			"<interleave>" +
+				"<element name='Value' a:help='Capture points value'><ref name='nonNegativeDecimal'/></element>" +
+				"<element name='MaxRange' a:help='Maximum attack range (in meters)'><ref name='nonNegativeDecimal'/></element>" +
+				"<element name='RepeatTime' a:help='Time between attacks (in milliseconds). The attack animation will be stretched to match this time'>" + // TODO: it shouldn't be stretched
+					"<data type='positiveInteger'/>" +
+				"</element>" +
+				Attack.prototype.bonusesSchema +
+				Attack.prototype.preferredClassesSchema +
+				Attack.prototype.restrictedClassesSchema +
+			"</interleave>" +
+		"</element>" +
+	"</optional>" +
 	"<optional>" +
 		"<element name='Charge'>" +
 			"<interleave>" +
@@ -166,9 +180,9 @@ Attack.prototype.Schema =
 				"<element name='Crush' a:help='Crush damage strength'><ref name='nonNegativeDecimal'/></element>" +
 				"<element name='MaxRange'><ref name='nonNegativeDecimal'/></element>" + // TODO: how do these work?
 				"<element name='MinRange'><ref name='nonNegativeDecimal'/></element>" +
-				bonusesSchema +
-				preferredClassesSchema +
-				restrictedClassesSchema +
+				Attack.prototype.bonusesSchema +
+				Attack.prototype.preferredClassesSchema +
+				Attack.prototype.restrictedClassesSchema +
 			"</interleave>" +
 		"</element>" +
 	"</optional>" +
@@ -179,9 +193,9 @@ Attack.prototype.Schema =
 				"<element name='Pierce' a:help='Pierce damage strength'><ref name='nonNegativeDecimal'/></element>" +
 				"<element name='Crush' a:help='Crush damage strength'><ref name='nonNegativeDecimal'/></element>" +
 				"<element name='MaxRange'><ref name='nonNegativeDecimal'/></element>" + // TODO: how do these work?
-				bonusesSchema +
-				preferredClassesSchema +
-				restrictedClassesSchema +
+				Attack.prototype.bonusesSchema +
+				Attack.prototype.preferredClassesSchema +
+				Attack.prototype.restrictedClassesSchema +
 			"</interleave>" +
 		"</element>" +
 	"</optional>";
@@ -198,6 +212,7 @@ Attack.prototype.GetAttackTypes = function()
 	if (this.template.Charge) ret.push("Charge");
 	if (this.template.Melee) ret.push("Melee");
 	if (this.template.Ranged) ret.push("Ranged");
+	if (this.template.Capture) ret.push("Capture");
 	return ret;
 };
 
@@ -243,26 +258,30 @@ Attack.prototype.CanAttack = function(target)
 
 	const targetClasses = cmpIdentity.GetClassesList();
 
-	for each (var type in this.GetAttackTypes())
+	for (let type of this.GetAttackTypes())
 	{
+		if (type == "Capture" && !QueryMiragedInterface(target, IID_Capturable))
+			continue;
+		if (type == "Slaughter" && targetClasses.indexOf("Domestic") == -1)
+			continue;
+
 		if (heightDiff > this.GetRange(type).max)
 			continue;
 
-		var canAttack = true;
-		var restrictedClasses = this.GetRestrictedClasses(type);
+		let restrictedClasses = this.GetRestrictedClasses(type);
+		if (!restrictedClasses.length)
+			return true;
 
-		for each (var targetClass in targetClasses)
+		let canAttack = true;
+		for (let targetClass of targetClasses)
 		{
-			if (restrictedClasses.indexOf(targetClass) != -1)
-			{
-				canAttack = false;
-				break;
-			}
+			if (restrictedClasses.indexOf(targetClass) == -1)
+				continue;
+			canAttack = false;
+			break;
 		}
 		if (canAttack)
-		{
 			return true;
-		}
 	}
 
 	return false;
@@ -280,53 +299,90 @@ Attack.prototype.GetPreference = function(target)
 	const targetClasses = cmpIdentity.GetClassesList();
 
 	var minPref = null;
-	for each (var type in this.GetAttackTypes())
+	for (let type of this.GetAttackTypes())
 	{
-		for each (var targetClass in targetClasses)
+		let preferredClasses = this.GetPreferredClasses(type);
+		for (let targetClass of targetClasses)
 		{
-			var pref = this.GetPreferredClasses(type).indexOf(targetClass);
+			let pref = preferredClasses.indexOf(targetClass);
+			if (pref === 0)
+				return pref;
 			if (pref != -1 && (minPref === null || minPref > pref))
-			{
 				minPref = pref;
-			}
 		}
 	}
 	return minPref;
 };
 
 /**
- * Return the type of the best attack.
- * TODO: this should probably depend on range, target, etc,
- * so we can automatically switch between ranged and melee
+ * Get the full range of attack using all available attack types.
  */
-Attack.prototype.GetBestAttack = function()
+Attack.prototype.GetFullAttackRange = function()
 {
-	return this.GetAttackTypes().pop();
+	let ret = {"min": Infinity, "max": 0};
+	for (let type of this.GetAttackTypes())
+	{
+		// Ignore the special attack "slaughter" dedicated to domestic animals
+		if (type == "Slaughter")
+			continue;
+		let range = this.GetRange(type);
+		if (range.min < ret.min)
+			ret.min = range.min;
+		if (range.max > ret.max)
+			ret.max = range.max;
+	}
+	return ret;
 };
 
-Attack.prototype.GetBestAttackAgainst = function(target)
+Attack.prototype.GetBestAttackAgainst = function(target, allowCapture)
 {
 	var cmpFormation = Engine.QueryInterface(target, IID_Formation);
 	if (cmpFormation)
-		return this.GetBestAttack();
+	{
+		// TODO: Formation against formation needs review
+		let best = ["Ranged", "Melee", "Capture"];
+		let types = this.GetAttackTypes();
+		for (let attack of best)
+			if (types.indexOf(attack) != -1)
+				return attack;
+		return undefined;
+	}
 
-	const cmpIdentity = Engine.QueryInterface(target, IID_Identity);
+	var cmpIdentity = Engine.QueryInterface(target, IID_Identity);
 	if (!cmpIdentity) 
 		return undefined;
 
-	const targetClasses = cmpIdentity.GetClassesList();
-	const isTargetClass = function (value, i, a) { return targetClasses.indexOf(value) != -1; };
-	const types = this.GetAttackTypes();
-	const attack = this;
-	const isAllowed = function (value, i, a) { return !attack.GetRestrictedClasses(value).some(isTargetClass); }
-	const isPreferred = function (value, i, a) { return attack.GetPreferredClasses(value).some(isTargetClass); }
-	const byPreference = function (a, b) { return (types.indexOf(a) + (isPreferred(a) ? types.length : 0) ) - (types.indexOf(b) + (isPreferred(b) ? types.length : 0) ); }
+
+	var targetClasses = cmpIdentity.GetClassesList();
+	var isTargetClass = function (className) { return targetClasses.indexOf(className) != -1; };
 
 	// Always slaughter domestic animals instead of using a normal attack
 	if (isTargetClass("Domestic") && this.template.Slaughter) 
 		return "Slaughter";
 
-	return types.filter(isAllowed).sort(byPreference).pop();
+	var attack = this;
+	var isAllowed = function (type) { return !attack.GetRestrictedClasses(type).some(isTargetClass); };
+
+	var types = this.GetAttackTypes().filter(isAllowed);
+
+	// check if the target is capturable
+	var captureIndex = types.indexOf("Capture");
+	if (captureIndex != -1)
+	{
+		let cmpCapturable = QueryMiragedInterface(target, IID_Capturable);
+
+		var cmpPlayer = QueryOwnerInterface(this.entity);
+		if (allowCapture && cmpPlayer && cmpCapturable && cmpCapturable.CanCapture(cmpPlayer.GetPlayerID()))
+			return "Capture";
+		// not captureable, so remove this attack
+		types.splice(captureIndex, 1);
+	}
+
+	var isPreferred = function (className) { return attack.GetPreferredClasses(className).some(isTargetClass); };
+	var byPreference = function (a, b) { return (types.indexOf(a) + (isPreferred(a) ? types.length : 0) ) - (types.indexOf(b) + (isPreferred(b) ? types.length : 0) ); };
+
+
+	return types.sort(byPreference).pop();
 };
 
 Attack.prototype.CompareEntitiesByPreference = function(a, b)
@@ -354,8 +410,6 @@ Attack.prototype.GetTimers = function(type)
 Attack.prototype.GetAttackStrengths = function(type)
 {
 	// Work out the attack values with technology effects
-	var self = this;
-
 	var template = this.template[type];
 	var splash = "";
 	if (!template)
@@ -364,15 +418,16 @@ Attack.prototype.GetAttackStrengths = function(type)
 		splash = "/Splash";
 	}
 	
-	var applyMods = function(damageType)
-	{
-		return ApplyValueModificationsToEntity("Attack/" + type + splash + "/" + damageType, +(template[damageType] || 0), self.entity);
-	};
-	
+	var applyMods = damageType =>
+		ApplyValueModificationsToEntity("Attack/" + type + splash + "/" + damageType, +(template[damageType] || 0), this.entity);
+
+	if (type == "Capture")
+		return { "value": applyMods("Value") };
+
 	return {
-		hack: applyMods("Hack"),
-		pierce: applyMods("Pierce"),
-		crush: applyMods("Crush")
+		"hack": applyMods("Hack"),
+		"pierce": applyMods("Pierce"),
+		"crush": applyMods("Crush")
 	};
 };
 
@@ -514,10 +569,30 @@ Attack.prototype.PerformAttack = function(type, target)
 		var cmpProjectileManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_ProjectileManager);
 		var id = cmpProjectileManager.LaunchProjectileAtPoint(this.entity, realTargetPosition, horizSpeed, gravity);
 
-		var playerId = Engine.QueryInterface(this.entity, IID_Ownership).GetOwner()
+		var playerId = Engine.QueryInterface(this.entity, IID_Ownership).GetOwner();
 		var cmpTimer = Engine.QueryInterface(SYSTEM_ENTITY, IID_Timer);
  		cmpTimer.SetTimeout(this.entity, IID_Attack, "MissileHit", timeToTarget*1000, {"type": type, "target": target, "position": realTargetPosition, "direction": missileDirection, "projectileId": id, "playerId":playerId});
 	}
+	else if (type == "Capture")
+	{
+		var multiplier = this.GetAttackBonus(type, target);
+		var cmpHealth = Engine.QueryInterface(target, IID_Health);
+		if (!cmpHealth || cmpHealth.GetHitpoints() == 0)
+			return;
+		multiplier *= cmpHealth.GetMaxHitpoints() / cmpHealth.GetHitpoints();
+
+		var cmpOwnership = Engine.QueryInterface(this.entity, IID_Ownership);
+		if (!cmpOwnership || cmpOwnership.GetOwner() == -1)
+			return;
+		var owner = cmpOwnership.GetOwner();
+		var cmpCapturable = Engine.QueryInterface(target, IID_Capturable);
+		if (!cmpCapturable || !cmpCapturable.CanCapture(owner))
+			return;
+		
+		var strength = this.GetAttackStrengths("Capture").value * multiplier;
+		if(cmpCapturable.Reduce(strength, owner))
+			Engine.PostMessage(target, MT_Attacked, {"attacker":this.entity, "target":target, "type":type, "damage":strength});
+	}
 	else
 	{
 		// Melee attack - hurt the target immediately
@@ -584,9 +659,9 @@ Attack.prototype.MissileHit = function(data, lateness)
 		var splashShape = this.template.Ranged.Splash.Shape;
 		var playersToDamage;
 		// If friendlyFire isn't enabled, get all player enemies to pass to "Damage.CauseSplashDamage".
-		if (friendlyFire == false)
+		if (friendlyFire == "false")
 		{
-			var cmpPlayer = Engine.QueryInterface(Engine.QueryInterface(SYSTEM_ENTITY, IID_PlayerManager).GetPlayerByID(data.playerId), IID_Player)
+			var cmpPlayer = QueryPlayerIDInterface(data.playerId);
 			playersToDamage = cmpPlayer.GetEnemies();
 		}
 		// Damage the units.
@@ -595,9 +670,9 @@ Attack.prototype.MissileHit = function(data, lateness)
 
 	if (this.testCollision(data.target, data.position, lateness))
 	{
-		data.attacker = this.entity
-		data.multiplier = this.GetAttackBonus(data.type, data.target)
-		data.strengths = this.GetAttackStrengths(data.type)
+		data.attacker = this.entity;
+		data.multiplier = this.GetAttackBonus(data.type, data.target);
+		data.strengths = this.GetAttackStrengths(data.type);
 		// Hit the primary target
 		Damage.CauseDamage(data);
 
@@ -608,8 +683,8 @@ Attack.prototype.MissileHit = function(data, lateness)
 	else
 	{
 		// If we didn't hit the main target look for nearby units
-		var cmpPlayer = Engine.QueryInterface(Engine.QueryInterface(SYSTEM_ENTITY, IID_PlayerManager).GetPlayerByID(data.playerId), IID_Player)
-		var ents = Damage.EntitiesNearPoint(data.position, targetPosition.horizDistanceTo(data.position) * 2, cmpPlayer.GetEnemies());
+		var cmpPlayer = QueryPlayerIDInterface(data.playerId);
+		var ents = Damage.EntitiesNearPoint(Vector2D.from3D(data.position), targetPosition.horizDistanceTo(data.position) * 2, cmpPlayer.GetEnemies());
 
 		for (var i = 0; i < ents.length; i++)
 		{
@@ -625,4 +700,21 @@ Attack.prototype.MissileHit = function(data, lateness)
 	}
 };
 
+Attack.prototype.OnValueModification = function(msg)
+{
+	if (msg.component != "Attack")
+		return;
+
+	var cmpUnitAI = Engine.QueryInterface(this.entity, IID_UnitAI);
+	if (!cmpUnitAI)
+		return;
+
+	for (var type of this.GetAttackTypes())
+		if (msg.valueNames.indexOf("Attack/"+type+"/MaxRange") !== -1)
+		{
+			cmpUnitAI.UpdateRangeQueries();
+			return;
+		}
+};
+
 Engine.RegisterComponentType(IID_Attack, "Attack", Attack);
