@@ -33,12 +33,13 @@ Cost.prototype.Schema =
 
 Cost.prototype.Init = function()
 {
+	this.populationCost = +this.template.Population;
 	this.populationBonus = +this.template.PopulationBonus;
 };
 
 Cost.prototype.GetPopCost = function()
 {
-	return +this.template.Population;
+	return this.populationCost;
 };
 
 Cost.prototype.GetPopBonus = function()
@@ -48,10 +49,10 @@ Cost.prototype.GetPopBonus = function()
 
 Cost.prototype.GetBuildTime = function()
 {
-	var cmpPlayer = QueryOwnerInterface(this.entity, IID_Player);
+	var cmpPlayer = QueryOwnerInterface(this.entity);
 	var buildTime = (+this.template.BuildTime) * cmpPlayer.cheatTimeMultiplier;
 	return ApplyValueModificationsToEntity("Cost/BuildTime", buildTime, this.entity);
-}
+};
 
 Cost.prototype.GetResourceCosts = function()
 {
@@ -87,19 +88,27 @@ Cost.prototype.OnValueModification = function(msg)
 	if (msg.component != "Cost")
 		return;
 
-	// foundations shouldn't give a pop bonus
-	var cmpFoundation = Engine.QueryInterface(this.entity, IID_Foundation)
+	// foundations shouldn't give a pop bonus and a pop cost
+	var cmpFoundation = Engine.QueryInterface(this.entity, IID_Foundation);
 	if (cmpFoundation)
 		return;
 
-	// update the population bonuses
-	var newPopBonus = ApplyValueModificationsToEntity("Cost/PopulationBonus",  +this.template.PopulationBonus, this.entity);
-	var popDifference = newPopBonus - this.populationBonus;
-	if (!popDifference)
-		return;
+	// update the population costs
+	var newPopCost = Math.round(ApplyValueModificationsToEntity("Cost/Population",  +this.template.Population, this.entity));
+	var popCostDifference = newPopCost - this.populationCost;
+	this.populationCost = newPopCost;
 
-	var cmpPlayer = QueryOwnerInterface(this.entity, IID_Player);
-	if (cmpPlayer)
+	// update the population bonuses
+	var newPopBonus = Math.round(ApplyValueModificationsToEntity("Cost/PopulationBonus",  +this.template.PopulationBonus, this.entity));
+	var popDifference = newPopBonus - this.populationBonus;
+	this.populationBonus = newPopBonus;
+
+	var cmpPlayer = QueryOwnerInterface(this.entity);
+	if (!cmpPlayer)
+		return;
+	if (popCostDifference)
+		cmpPlayer.AddPopulation(popCostDifference);
+	if (popDifference)
 		cmpPlayer.AddPopulationBonuses(popDifference);
 };
 
