@@ -8,7 +8,7 @@ ProductionQueue.prototype.Schema =
 	"<a:example>" +
 		"<BatchTimeModifier>0.7</BatchTimeModifier>" +
 		"<Entities datatype='tokens'>" +
-			"\n    units/{civ}_support_female_citizen\n    units/{civ}_support_trader\n    units/celt_infantry_spearman_b\n  " +
+			"\n    units/{civ}_support_female_citizen\n    units/{civ}_support_trader\n    units/athen_infantry_spearman_b\n  " +
 		"</Entities>" +
 	"</a:example>" +
 	"<element name='BatchTimeModifier' a:help='Modifier that influences the time benefit for batch training'>" +
@@ -92,33 +92,32 @@ ProductionQueue.prototype.CalculateEntitiesList = function()
 	this.entitiesList = [];
 	if (!this.template.Entities)
 		return;
-	
+
 	var string = this.template.Entities._string;
 	if (!string)
 		return;
-	
+
 	// Replace the "{civ}" codes with this entity's civ ID
-	var cmpIdentity = Engine.QueryInterface(this.entity, IID_Identity);
-	if (cmpIdentity)
-		string = string.replace(/\{civ\}/g, cmpIdentity.GetCiv());
-	
-	var entitiesList = string.split(/\s+/);
-	
-	// Remove disabled entities
-	var cmpPlayer = QueryOwnerInterface(this.entity, IID_Player);
+	var cmpTemplateManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_TemplateManager);
+	var cmpPlayer = QueryOwnerInterface(this.entity);
+	if (!cmpPlayer)
+		return;
+
+	var entitiesList = string.replace(/\{civ\}/g, cmpPlayer.GetCiv()).split(/\s+/);
+
+	// filter out disabled and invalid entities
 	var disabledEntities = cmpPlayer.GetDisabledTemplates();
-	
-	for (var i = entitiesList.length - 1; i >= 0; --i)
-		if (disabledEntities[entitiesList[i]])
-			entitiesList.splice(i, 1);
-	
+	entitiesList = entitiesList.filter(
+		function(v) { return !disabledEntities[v] && cmpTemplateManager.TemplateExists(v); }
+	);
+
 	// check if some templates need to show their advanced or elite version
 	var upgradeTemplate = function(templateName)
 	{
 		var template = cmpTemplateManager.GetTemplate(templateName);
 		while (template && template.Promotion !== undefined)
 		{
-			var requiredXp = ApplyValueModificationsToTemplate("Promotion/RequiredXp", +template.Promotion.RequiredXp, playerID, template);
+			var requiredXp = ApplyValueModificationsToTemplate("Promotion/RequiredXp", +template.Promotion.RequiredXp, cmpPlayer.GetPlayerID(), template);
 			if (requiredXp > 0)
 				break;
 			templateName = template.Promotion.Entity;
@@ -127,15 +126,12 @@ ProductionQueue.prototype.CalculateEntitiesList = function()
 		return templateName;
 	};
 
-	var cmpTemplateManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_TemplateManager);
-	var playerID = cmpPlayer.GetPlayerID();
-	for each (var templateName in entitiesList)
+	for (let templateName of entitiesList)
 		this.entitiesList.push(upgradeTemplate(templateName));
-	for each (var item in this.queue)
-	{
+
+	for (let item of this.queue)
 		if (item.unitTemplate)
 			item.unitTemplate = upgradeTemplate(item.unitTemplate);
-	}
 };
 
 /*
@@ -154,15 +150,24 @@ ProductionQueue.prototype.GetTechnologiesList = function()
 	if (!cmpTechnologyManager)
 		return [];
 	
+	var cmpPlayer = QueryOwnerInterface(this.entity);
+	var cmpIdentity = Engine.QueryInterface(this.entity, IID_Identity);
+	if (!cmpPlayer || !cmpIdentity || cmpPlayer.GetCiv() != cmpIdentity.GetCiv())
+		return [];
+
 	var techs = string.split(/\s+/);
 	var techList = [];
-	var superseded = {}; // Stores the tech which supersedes the key 
+	var superseded = {}; // Stores the tech which supersedes the key
+
+	var disabledTechnologies = cmpPlayer.GetDisabledTechnologies();
 	
 	// Add any top level technologies to an array which corresponds to the displayed icons
 	// Also store what a technology is superceded by in the superceded object {"tech1":"techWhichSupercedesTech1", ...}
 	for (var i in techs)
 	{
 		var tech = techs[i];
+		if (disabledTechnologies && disabledTechnologies[tech])
+			continue;
 		var template = cmpTechnologyManager.GetTechnologyTemplate(tech);
 		if (!template.supersedes || techs.indexOf(template.supersedes) === -1)
 			techList.push(tech);
@@ -182,7 +187,7 @@ ProductionQueue.prototype.GetTechnologiesList = function()
 		techList[i] = tech;
 	}
 	
-	var ret = []
+	var ret = [];
 	
 	// This inserts the techs into the correct positions to line up the technology pairs
 	for (var i = 0; i < techList.length; i++)
@@ -215,11 +220,11 @@ ProductionQueue.prototype.IsTechnologyResearchedOrInProgress = function(tech)
 	if (template.top)
 	{
 		return (cmpTechnologyManager.IsTechnologyResearched(template.top) || cmpTechnologyManager.IsInProgress(template.top)
-			|| cmpTechnologyManager.IsTechnologyResearched(template.bottom) || cmpTechnologyManager.IsInProgress(template.bottom))
+			|| cmpTechnologyManager.IsTechnologyResearched(template.bottom) || cmpTechnologyManager.IsInProgress(template.bottom));
 	}
 	else
 	{
-		return (cmpTechnologyManager.IsTechnologyResearched(tech) || cmpTechnologyManager.IsInProgress(tech))
+		return (cmpTechnologyManager.IsTechnologyResearched(tech) || cmpTechnologyManager.IsInProgress(tech));
 	}
 };
 
@@ -231,7 +236,7 @@ ProductionQueue.prototype.AddBatch = function(templateName, type, count, metadat
 	// TODO: there should probably be a limit on the number of queued batches
 	// TODO: there should be a way for the GUI to determine whether it's going
 	// to be possible to add a batch (based on resource costs and length limits)
-	var cmpPlayer = QueryOwnerInterface(this.entity, IID_Player);
+	var cmpPlayer = QueryOwnerInterface(this.entity);
 
 	if (this.queue.length < MAX_QUEUE_SIZE)
 	{
@@ -239,8 +244,8 @@ ProductionQueue.prototype.AddBatch = function(templateName, type, count, metadat
 		if (type == "unit")
 		{
 			// Find the template data so we can determine the build costs
-			var cmpTempMan = Engine.QueryInterface(SYSTEM_ENTITY, IID_TemplateManager);
-			var template = cmpTempMan.GetTemplate(templateName);
+			var cmpTemplateManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_TemplateManager);
+			var template = cmpTemplateManager.GetTemplate(templateName);
 			if (!template)
 				return;
 			if (template.Promotion)
@@ -270,7 +275,7 @@ ProductionQueue.prototype.AddBatch = function(templateName, type, count, metadat
 				totalCosts[r] = Math.floor(count * costs[r]);
 			}
 
-			var population = +template.Cost.Population;
+			var population = ApplyValueModificationsToTemplate("Cost/Population",  +template.Cost.Population, cmpPlayer.GetPlayerID(), template);
 
 			// TrySubtractResources should report error to player (they ran out of resources)
 			if (!cmpPlayer.TrySubtractResources(totalCosts))
@@ -304,11 +309,11 @@ ProductionQueue.prototype.AddBatch = function(templateName, type, count, metadat
 		else if (type == "technology")
 		{
 			// Load the technology template
-			var cmpTechTempMan = Engine.QueryInterface(SYSTEM_ENTITY, IID_TechnologyTemplateManager);
-			var template = cmpTechTempMan.GetTemplate(templateName);
+			var cmpTechnologyTemplateManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_TechnologyTemplateManager);
+			var template = cmpTechnologyTemplateManager.GetTemplate(templateName);
 			if (!template)
 				return;
-			var cmpPlayer = QueryOwnerInterface(this.entity, IID_Player);
+			var cmpPlayer = QueryOwnerInterface(this.entity);
 			var time = template.researchTime * cmpPlayer.GetCheatTimeMultiplier();
 
 			var cost = {};
@@ -385,13 +390,13 @@ ProductionQueue.prototype.RemoveBatch = function(id)
 
 		// Now we've found the item to remove
 		
-		var cmpPlayer = QueryPlayerIDInterface(item.player, IID_Player);
+		var cmpPlayer = QueryPlayerIDInterface(item.player);
 
 		// Update entity count in the EntityLimits component
 		if (item.unitTemplate)
 		{
-			var cmpTempMan = Engine.QueryInterface(SYSTEM_ENTITY, IID_TemplateManager);
-			var template = cmpTempMan.GetTemplate(item.unitTemplate);
+			var cmpTemplateManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_TemplateManager);
+			var template = cmpTemplateManager.GetTemplate(item.unitTemplate);
 			if (template.TrainingRestrictions)
 			{
 				var unitCategory = template.TrainingRestrictions.Category;
@@ -473,7 +478,7 @@ ProductionQueue.prototype.ResetQueue = function()
  */
 ProductionQueue.prototype.GetBatchTime = function(batchSize)
 {
-	var cmpPlayer = QueryOwnerInterface(this.entity, IID_Player);
+	var cmpPlayer = QueryOwnerInterface(this.entity);
 
 	var batchTimeModifier = ApplyValueModificationsToEntity("ProductionQueue/BatchTimeModifier", +this.template.BatchTimeModifier, this.entity);
 
@@ -486,7 +491,7 @@ ProductionQueue.prototype.OnOwnershipChanged = function(msg)
 	if (msg.from != -1)
 	{
 		// Unset flag that previous owner's training may be blocked
-		var cmpPlayer = QueryPlayerIDInterface(msg.from, IID_Player);
+		var cmpPlayer = QueryPlayerIDInterface(msg.from);
 		if (cmpPlayer && this.queue.length > 0)
 			cmpPlayer.UnBlockTraining();
 	}
@@ -501,6 +506,11 @@ ProductionQueue.prototype.OnOwnershipChanged = function(msg)
 	this.ResetQueue();
 };
 
+ProductionQueue.prototype.OnCivChanged = function()
+{
+	this.CalculateEntitiesList();
+};
+
 ProductionQueue.prototype.OnDestroy = function()
 {
 	// Reset the queue to refund any resources
@@ -560,12 +570,12 @@ ProductionQueue.prototype.SpawnUnits = function(templateName, count, metadata)
 	for (var i = 0; i < count; ++i)
 	{
 		var ent = this.entityCache[0];
-		var cmpNewOwnership = Engine.QueryInterface(ent, IID_Ownership);
-		cmpNewOwnership.SetOwner(cmpOwnership.GetOwner());
 
 		if (cmpAutoGarrison && cmpAutoGarrison.PerformGarrison(ent))
 		{
-			var cmpUnitAI = Engine.QueryInterface(ent, IID_UnitAI);
+			let cmpNewOwnership = Engine.QueryInterface(ent, IID_Ownership);
+			cmpNewOwnership.SetOwner(cmpOwnership.GetOwner());
+			let cmpUnitAI = Engine.QueryInterface(ent, IID_UnitAI);
 			cmpUnitAI.Autogarrison(this.entity);
 		}
 		else
@@ -579,7 +589,9 @@ ProductionQueue.prototype.SpawnUnits = function(templateName, count, metadata)
 			else
 			{
 				// Successfully spawned
-				var cmpNewPosition = Engine.QueryInterface(ent, IID_Position);
+				let cmpNewOwnership = Engine.QueryInterface(ent, IID_Ownership);
+				cmpNewOwnership.SetOwner(cmpOwnership.GetOwner());
+				let cmpNewPosition = Engine.QueryInterface(ent, IID_Position);
 				cmpNewPosition.JumpTo(pos.x, pos.z);
 				// TODO: what direction should they face in?
 				spawnedEnts.push(ent);
@@ -645,7 +657,7 @@ ProductionQueue.prototype.ProgressTimeout = function(data)
 	// until we've used up all the time (so that we work accurately
 	// with items that take fractions of a second)
 	var time = g_ProgressInterval;
-	var cmpPlayer = QueryOwnerInterface(this.entity, IID_Player);
+	var cmpPlayer = QueryOwnerInterface(this.entity);
 
 	while (time > 0 && this.queue.length)
 	{
@@ -655,6 +667,10 @@ ProductionQueue.prototype.ProgressTimeout = function(data)
 			// If the item is a unit then do population checks
 			if (item.unitTemplate)
 			{
+				// If something change population cost
+				var template = Engine.QueryInterface(SYSTEM_ENTITY, IID_TemplateManager).GetTemplate(item.unitTemplate);
+				item.population = ApplyValueModificationsToTemplate("Cost/Population",  +template.Cost.Population, item.player, template);
+				
 				// Batch's training hasn't started yet.
 				// Try to reserve the necessary population slots
 				item.neededSlots = cmpPlayer.TryReservePopulationSlots(item.population * item.count);
@@ -723,7 +739,7 @@ ProductionQueue.prototype.ProgressTimeout = function(data)
 				
 				if (!this.spawnNotified)
 				{
-					var cmpPlayer = QueryOwnerInterface(this.entity, IID_Player);
+					var cmpPlayer = QueryOwnerInterface(this.entity);
 					var notification = {"players": [cmpPlayer.GetPlayerID()], "message": "Can't find free space to spawn trained units" };
 					var cmpGUIInterface = Engine.QueryInterface(SYSTEM_ENTITY, IID_GuiInterface);
 					cmpGUIInterface.PushNotification(notification);
