@@ -217,7 +217,7 @@ for (var i = 0; i < numPlayers; i++)
 	//create docks
 	var dockLocation = getTIPIADBON([ix, iz], [mapSize / 2, mapSize / 2], [-3 , 2.6], 0.5, 3);
 	if (dockLocation !== undefined)
-		placeObject(dockLocation[0], dockLocation[1], "structures/" + g_MapSettings.PlayerData[id-1].Civ + "_dock", id, playerAngle[i] + PI);
+		placeObject(dockLocation[0], dockLocation[1], "structures/" + getCivCode(id-1) + "_dock", id, playerAngle[i] + PI);
 }
 
 var landAreas = [];
@@ -240,7 +240,7 @@ for (var i = 0; i < numIslands; ++i)
 	if (!landAreaLen)
 		break;
 	
-	chosenPoint = landAreas[randInt(0, landAreaLen)];
+	chosenPoint = landAreas[randInt(landAreaLen)];
 	
 	// create big islands
 	placer = new ChainPlacer(floor(scaleByMapSize(4, 8)), floor(scaleByMapSize(8, 14)), floor(scaleByMapSize(25, 60)), 0.07, chosenPoint[0], chosenPoint[1], scaleByMapSize(30,70));
@@ -256,16 +256,16 @@ for (var i = 0; i < numIslands; ++i)
 		avoidClasses(clLand, 3, clPlayer, 3),
 		1, 1
 	);
-	if (newIsland !== undefined)
+	if (newIsland && newIsland.length)
 	{
-		var temp = []
+		var n = 0;
 		for (var j = 0; j < landAreaLen; ++j)
 		{
 			var x = landAreas[j][0], z = landAreas[j][1];
 			if (playerConstraint.allows(x, z) && landConstraint.allows(x, z))
-					temp.push([x, z]);
+				landAreas[n++] = landAreas[j];
 		}
-		landAreas = temp;
+		landAreas.length = n;
 	}
 }
 
