@@ -24,7 +24,7 @@ setFogFactor(0.35);
 setFogThickness(0.19);
 
 // water
-setWaterColour(0.501961, 0.501961, 0.501961);
+setWaterColor(0.501961, 0.501961, 0.501961);
 setWaterTint(0.25098, 0.501961, 0.501961);
 setWaterWaviness(0.5);
 setWaterType("clap");
@@ -276,7 +276,7 @@ Takes
 	heightmap		Optional, default is g_Map.height, an array of (map width) arrays of (map depth) floats
 	maxTries		Optional, default is 1000, an integer, how often random player distributions are rolled to be compared
 	minDistToBorder	Optional, default is 20, an integer, how far start locations have to be 
-	numberOfPlayers	Optional, default is g_MapSettings.PlayerData.length, an integer, how many start locations should be placed
+	numberOfPlayers	Optional, default is getNumPlayers, an integer, how many start locations should be placed
 Returns
 	An array of 2D points (arrays of length 2)
 */
@@ -284,7 +284,7 @@ function getStartLocationsByHeightmap(hightRange, maxTries, minDistToBorder, num
 {
 	maxTries = (maxTries || 1000);
 	minDistToBorder = (minDistToBorder || 20);
-	numberOfPlayers = (numberOfPlayers || g_MapSettings.PlayerData.length);
+	numberOfPlayers = (numberOfPlayers || getNumPlayers());
 	heightmap = (heightmap || g_Map.height);
 	
 	var validStartLocTiles = [];
@@ -533,7 +533,7 @@ function rectangularSmoothToHeight(center, dx, dy, targetHeight, strength, heigh
 ////////////////
 
 // Set target min and max height depending on map size to make average steepness about the same on all map sizes
-var heightRange = {'min': MIN_HEIGHT * (g_Map.size + 512) / 1024, 'max': MAX_HEIGHT * (g_Map.size + 512) / 1024, 'avg': ((MIN_HEIGHT * (g_Map.size + 512) / 1024)+(MAX_HEIGHT * (g_Map.size + 512) / 1024))/2};
+var heightRange = {'min': MIN_HEIGHT * (g_Map.size + 512) / 8192, 'max': MAX_HEIGHT * (g_Map.size + 512) / 8192, 'avg': (MIN_HEIGHT * (g_Map.size + 512) +MAX_HEIGHT * (g_Map.size + 512))/16384};
 
 // Set average water coverage
 var averageWaterCoverage = 1/5; // NOTE: Since erosion is not predictable actual water coverage might vary much with the same values
