@@ -1,3 +1,6 @@
+const g_IsReplay = Engine.IsVisualReplay();
+const g_GameSpeeds = prepareForDropdown(g_Settings ? g_Settings.GameSpeeds.filter(speed => !speed.ReplayOnly || g_IsReplay) : undefined);
+
 // Network Mode
 var g_IsNetworked = false;
 
@@ -13,9 +16,6 @@ var g_Players = [];
 // Cache the useful civ data
 var g_CivData = {};
 
-var g_GameSpeeds = {};
-var g_CurrentSpeed;
-
 var g_PlayerAssignments = { "local": { "name": translate("You"), "player": 1 } };
 
 // Cache dev-mode settings that are frequently or widely used
@@ -44,6 +44,9 @@ var g_Disconnected = false; // Lost connection to server
 // Holds player states from the last tick
 var g_CachedLastStates = "";
 
+// Top coordinate of the research list
+var g_ResearchListTop = 4;
+
 // Colors to flash when pop limit reached
 const DEFAULT_POPULATION_COLOR = "white";
 const POPULATION_ALERT_COLOR = "orange";
@@ -68,33 +71,23 @@ function GetSimState()
 
 function GetEntityState(entId)
 {
-	if (!(entId in g_EntityStates))
-	{
-		var entState = Engine.GuiInterfaceCall("GetEntityState", entId);
-		if (entState)
-			entState.extended = false;
-		g_EntityStates[entId] = entState;
-	}
+	if (!g_EntityStates[entId])
+		g_EntityStates[entId] = Engine.GuiInterfaceCall("GetEntityState", entId);
 
 	return g_EntityStates[entId];
 }
 
 function GetExtendedEntityState(entId)
 {
-	if (entId in g_EntityStates)
-		var entState = g_EntityStates[entId];
-	else
-		var entState = Engine.GuiInterfaceCall("GetEntityState", entId);
-
+	let entState = GetEntityState(entId);
 	if (!entState || entState.extended)
 		return entState;
 
-	var extension = Engine.GuiInterfaceCall("GetExtendedEntityState", entId);
-	for (var prop in extension)
+	let extension = Engine.GuiInterfaceCall("GetExtendedEntityState", entId);
+	for (let prop in extension)
 		entState[prop] = extension[prop];
 	entState.extended = true;
 	g_EntityStates[entId] = entState;
-
 	return entState;
 }
 
@@ -141,9 +134,15 @@ function GetTechnologyData(technologyName)
 	return g_TechnologyData[technologyName];
 }
 
-// Init
 function init(initData, hotloadData)
 {
+	if (!g_Settings)
+	{
+		Engine.EndGame();
+		Engine.SwitchGuiPage("page_pregame.xml");
+		return;
+	}
+
 	if (initData)
 	{
 		g_IsNetworked = initData.isNetworked; // Set network mode
@@ -162,6 +161,9 @@ function init(initData, hotloadData)
 	}
 	else // Needed for autostart loading option
 	{
+		if (g_IsReplay)
+			g_PlayerAssignments.local.player = -1;
+
 		g_Players = getPlayerData(null);
 	}
 
@@ -170,35 +172,16 @@ function init(initData, hotloadData)
 	g_CivData["gaia"] = { "Code": "gaia", "Name": translate("Gaia") };
 
 	if (Engine.GetPlayerID() <= 0)
-	{
 		g_IsObserver = true;
-		// Hide stuff observers don't use.
-		Engine.GetGUIObjectByName("food").hidden = true;
-		Engine.GetGUIObjectByName("wood").hidden = true;
-		Engine.GetGUIObjectByName("stone").hidden = true;
-		Engine.GetGUIObjectByName("metal").hidden = true;
-		Engine.GetGUIObjectByName("population").hidden = true;
-		Engine.GetGUIObjectByName("diplomacyButton1").hidden = true;
-		Engine.GetGUIObjectByName("tradeButton1").hidden = true;
-		Engine.GetGUIObjectByName("menuResignButton").enabled = false;
-		Engine.GetGUIObjectByName("pauseButton").enabled = false;
-		Engine.GetGUIObjectByName("observerText").hidden = false;
-	}
-	else
-	{
-		// TODO: Get a civ icon for gaia/observers.
-		Engine.GetGUIObjectByName("civIcon").sprite = "stretched:" + g_CivData[g_Players[Engine.GetPlayerID()].civ].Emblem;
-		Engine.GetGUIObjectByName("civIcon").tooltip = g_CivData[g_Players[Engine.GetPlayerID()].civ].Name;
-	}
 
-	g_GameSpeeds = initGameSpeeds();
-	g_CurrentSpeed = Engine.GetSimRate();
+	updateTopPanel();
+
 	var gameSpeed = Engine.GetGUIObjectByName("gameSpeed");
-	gameSpeed.list = g_GameSpeeds.names;
-	gameSpeed.list_data = g_GameSpeeds.speeds;
-	var idx = g_GameSpeeds.speeds.indexOf(g_CurrentSpeed);
-	gameSpeed.selected = idx != -1 ? idx : g_GameSpeeds["default"];
-	gameSpeed.onSelectionChange = function() { changeGameSpeed(+this.list_data[this.selected]); }
+	gameSpeed.list = g_GameSpeeds.Title;
+	gameSpeed.list_data = g_GameSpeeds.Speed;
+	var gameSpeedIdx = g_GameSpeeds.Speed.indexOf(Engine.GetSimRate());
+	gameSpeed.selected = gameSpeedIdx != -1 ? gameSpeedIdx : g_GameSpeeds.Default;
+	gameSpeed.onSelectionChange = function() { changeGameSpeed(+this.list_data[this.selected]); };
 	initMenuPosition(); // set initial position
 
 	// Populate player selection dropdown
@@ -220,12 +203,12 @@ function init(initData, hotloadData)
 		Engine.GetGUIObjectByName("menuExitButton").enabled = false;
 
 	if (hotloadData)
-	{
 		g_Selection.selected = hotloadData.selection;
-	}
+
 	// Starting for the first time:
 	initMusic();
-	if (!g_IsObserver){
+	if (!g_IsObserver)
+	{
 		var civMusic = g_CivData[g_Players[Engine.GetPlayerID()].civ].Music;
 		global.music.storeTracks(civMusic);
 	}
@@ -237,17 +220,45 @@ function init(initData, hotloadData)
 	// the initial camera view) and a minute (when the profiler will
 	// have settled down if framerates as very low), to give some
 	// extremely rough indications of performance
-	setTimeout(function() { reportPerformance(5); }, 5000);
-	setTimeout(function() { reportPerformance(60); }, 60000);
+	//
+	// DISABLED: this information isn't currently useful for anything much,
+	// and it generates a massive amount of data to transmit and store
+	//setTimeout(function() { reportPerformance(5); }, 5000);
+	//setTimeout(function() { reportPerformance(60); }, 60000);
 }
 
 function selectViewPlayer(playerID)
 {
 	Engine.SetPlayerID(playerID);
-	if (playerID > 0) {
+	updateTopPanel();
+}
+
+function updateTopPanel()
+{
+	var playerID = Engine.GetPlayerID();
+	var isPlayer =  playerID > 0;
+	if (isPlayer)
+	{
+		var civName = g_CivData[g_Players[playerID].civ].Name;
 		Engine.GetGUIObjectByName("civIcon").sprite = "stretched:" + g_CivData[g_Players[playerID].civ].Emblem;
-		Engine.GetGUIObjectByName("civIcon").tooltip = g_CivData[g_Players[playerID].civ].Name;
+		Engine.GetGUIObjectByName("civIconOverlay").tooltip = sprintf(translate("%(civ)s - Structure Tree"), {"civ": civName});
 	}
+	
+	// Hide stuff gaia/observers don't use.
+	Engine.GetGUIObjectByName("food").hidden = !isPlayer;
+	Engine.GetGUIObjectByName("wood").hidden = !isPlayer;
+	Engine.GetGUIObjectByName("stone").hidden = !isPlayer;
+	Engine.GetGUIObjectByName("metal").hidden = !isPlayer;
+	Engine.GetGUIObjectByName("population").hidden = !isPlayer;
+	Engine.GetGUIObjectByName("civIcon").hidden = !isPlayer;
+	Engine.GetGUIObjectByName("diplomacyButton1").hidden = !isPlayer;
+	Engine.GetGUIObjectByName("tradeButton1").hidden = !isPlayer;
+	Engine.GetGUIObjectByName("observerText").hidden = playerID >= 0;
+
+	// Disable stuff observers shouldn't use
+	var isActive = isPlayer && GetSimState().players[playerID].state == "active";
+	Engine.GetGUIObjectByName("pauseButton").enabled = isActive || !g_IsNetworked;
+	Engine.GetGUIObjectByName("menuResignButton").enabled = isActive;
 }
 
 function reportPerformance(time)
@@ -281,7 +292,9 @@ function resignGame(leaveGameAfterResign)
 		"type": "defeat-player",
 		"playerId": Engine.GetPlayerID()
 	});
-
+	
+	updateTopPanel();
+	
 	global.music.setState(global.music.states.DEFEAT);
 	
 	// Resume the game if not resigning.
@@ -327,19 +340,25 @@ function leaveGame(willRejoin)
 		}
 	}
 
+	let summary = {
+		"timeElapsed" : extendedSimState.timeElapsed,
+		"playerStates": extendedSimState.players,
+		"players": g_Players,
+		"mapSettings": Engine.GetMapSettings(),
+	}
+
+	if (!g_IsReplay)
+		Engine.SaveReplayMetadata(JSON.stringify(summary));
+
 	stopAmbient();
 	Engine.EndGame();
 
 	if (g_IsController && Engine.HasXmppClient())
 		Engine.SendUnregisterGame();
 
-	Engine.SwitchGuiPage("page_summary.xml", {
-							"gameResult"  : gameResult,
-							"timeElapsed" : extendedSimState.timeElapsed,
-							"playerStates": extendedSimState.players,
-							"players": g_Players,
-							"mapSettings": mapSettings
-						 });
+	summary.gameResult = gameResult;
+	summary.isReplay = g_IsReplay;
+	Engine.SwitchGuiPage("page_summary.xml", summary);
 }
 
 // Return some data that we'll use when hotloading this file after changes
@@ -377,20 +396,21 @@ function restoreSavedGameData(data)
 	updateGroups();
 }
 
-var lastTickTime = new Date;
-var lastXmppClientPoll = Date.now();
+var lastTickTime = new Date();
 
 /**
  * Called every frame.
  */
 function onTick()
 {
-	var now = new Date;
-	var tickLength = new Date - lastTickTime;
+	if (!g_Settings)
+		return;
+
+	var now = new Date();
+	var tickLength = new Date() - lastTickTime;
 	lastTickTime = now;
 
 	checkPlayerState();
-
 	while (true)
 	{
 		var message = Engine.PollNetworkClient();
@@ -420,7 +440,7 @@ function onTick()
 	// Animate menu
 	updateMenuPosition(tickLength);
 
-	// When training is blocked, flash population (alternates colour every 500msec)
+	// When training is blocked, flash population (alternates color every 500msec)
 	if (g_IsTrainingBlocked && (Date.now() % 1000) < 500)
 		Engine.GetGUIObjectByName("resourcePop").textcolor = POPULATION_ALERT_COLOR;
 	else
@@ -452,9 +472,9 @@ function checkPlayerState()
 	if (playerState.state == "active")
 		return;
 
-	// We can't resign once the game is over.
-	Engine.GetGUIObjectByName("menuResignButton").enabled = false;
-
+	// Disable resign and pause buttons (we can't resign once the game is over)
+	updateTopPanel();
+	
 	// Make sure nothing is open to avoid stacking.
 	closeMenu();
 	closeOpenDialogs();
@@ -495,10 +515,7 @@ function changeGameSpeed(speed)
 {
 	// For non-networked games only
 	if (!g_IsNetworked)
-	{
 		Engine.SetSimRate(speed);
-		g_CurrentSpeed = speed;
-	}
 }
 
 /**
@@ -531,12 +548,12 @@ function onSimulationUpdate()
 	updatePlayerDisplay();
 	updateSelectionDetails();
 	updateBuildingPlacementPreview();
-	updateTimeElapsedCounter();
 	updateTimeNotifications();
+	if (!g_IsObserver)
+		updateResearchDisplay();
 
 	if (!g_IsObserver && !g_GameEnded)
 	{
-		updateResearchDisplay();
 		// Update music state on basis of battle state.
 		var battleState = Engine.GuiInterfaceCall("GetBattleState", Engine.GetPlayerID());
 		if (battleState)
@@ -544,6 +561,16 @@ function onSimulationUpdate()
 	}
 }
 
+function onReplayFinished()
+{
+	closeMenu();
+	closeOpenDialogs();
+	pauseGame();
+	var btCaptions = [translateWithContext("replayFinished", "No"), translateWithContext("replayFinished", "Yes")];
+	var btCode = [resumeGame, leaveGame];
+	messageBox(400, 200, translateWithContext("replayFinished", "The replay has finished. Do you want to quit?"), translateWithContext("replayFinished","Confirmation"), 0, btCaptions, btCode);
+}
+
 /**
 * updates a status bar on the GUI
 * nameOfBar: name of the bar
@@ -585,8 +612,7 @@ function updateGUIStatusBar(nameOfBar, points, maxPoints, direction)
 
 function updateHero()
 {
-	var simState = GetSimState();
-	var playerState = simState.players[Engine.GetPlayerID()];
+	var playerState = GetSimState().players[Engine.GetPlayerID()];
 	var unitHeroPanel = Engine.GetGUIObjectByName("unitHeroPanel");
 	var heroButton = Engine.GetGUIObjectByName("unitHeroButton");
 
@@ -617,26 +643,11 @@ function updateHero()
 	var healthLabel = "[font=\"sans-bold-13\"]" + translate("Health:") + "[/font]";
 	tooltip += "\n" + sprintf(translate("%(label)s %(current)s / %(max)s"), { label: healthLabel, current: heroState.hitpoints, max: heroState.maxHitpoints });
 	if (heroState.attack)
-	{
-		var attackLabel = "[font=\"sans-bold-13\"]" + getAttackTypeLabel(heroState.attack.type) + "[/font]";
-		if (heroState.attack.type == "Ranged")
-			// Show max attack range if ranged attack, also convert to tiles (4m per tile)
-			tooltip += "\n" + sprintf(
-				translate("%(attackLabel)s %(details)s, %(rangeLabel)s %(range)s"),
-				{
-					attackLabel: attackLabel,
-					details: damageTypeDetails(heroState.attack),
-					rangeLabel: "[font=\"sans-bold-13\"]" + translate("Range:") + "[/font]",
-					range: Math.round(heroState.attack.maxRange) + " [font=\"sans-10\"][color=\"orange\"]" + translate("meters") + "[/color][/font]",
-				}
-			);
-		else
-			tooltip += "\n" + sprintf(translate("%(label)s %(details)s"), { label: attackLabel, details: damageTypeDetails(heroState.attack) });
-	}
+		tooltip += "\n" + getAttackTooltip(heroState);
 
-	var armorLabel = "[font=\"sans-bold-13\"]" + translate("Armor:") + "[/font]";
-	tooltip += "\n" + sprintf(translate("%(label)s %(details)s"), { label: armorLabel, details: damageTypeDetails(heroState.armour) });
-	tooltip += "\n" + template.tooltip;
+	tooltip += "\n" + getArmorTooltip(heroState.armour);
+	if (template.tooltip)
+		tooltip += "\n" + template.tooltip;
 
 	heroButton.tooltip = tooltip;
 	
@@ -676,44 +687,40 @@ function updateGroups()
 
 function updateDebug()
 {
-	var simState = GetSimState();
-	var debug = Engine.GetGUIObjectByName("debug");
+	let debug = Engine.GetGUIObjectByName("debug");
 
-	if (Engine.GetGUIObjectByName("devDisplayState").checked)
-	{
-		debug.hidden = false;
-	}
-	else
+	if (!Engine.GetGUIObjectByName("devDisplayState").checked)
 	{
 		debug.hidden = true;
 		return;
 	}
 
-	var conciseSimState = deepcopy(simState);
-	conciseSimState.players = "<<<omitted>>>";
-	var text = "simulation: " + uneval(conciseSimState);
+	debug.hidden = false;
 
-	var selection = g_Selection.toList();
+	let conciseSimState = deepcopy(GetSimState());
+	conciseSimState.players = "<<<omitted>>>";
+	let text = "simulation: " + uneval(conciseSimState);
+
+	let selection = g_Selection.toList();
 	if (selection.length)
 	{
-		var entState = GetExtendedEntityState(selection[0]);
+		let entState = GetExtendedEntityState(selection[0]);
 		if (entState)
 		{
-			var template = GetTemplateData(entState.template);
+			let template = GetTemplateData(entState.template);
 			text += "\n\nentity: {\n";
-			for (var k in entState)
+			for (let k in entState)
 				text += "  "+k+":"+uneval(entState[k])+"\n";
 			text += "}\n\ntemplate: " + uneval(template);
 		}
 	}
 
-	debug.caption = text;
+	debug.caption = text.replace(/\[/g, "\\[");
 }
 
 function updatePlayerDisplay()
 {
-	var simState = GetSimState();
-	var playerState = simState.players[Engine.GetPlayerID()];
+	var playerState = GetSimState().players[Engine.GetPlayerID()];
 	if (!playerState)
 		return;
 
@@ -722,6 +729,8 @@ function updatePlayerDisplay()
 	Engine.GetGUIObjectByName("resourceStone").caption = Math.floor(playerState.resourceCounts.stone);
 	Engine.GetGUIObjectByName("resourceMetal").caption = Math.floor(playerState.resourceCounts.metal);
 	Engine.GetGUIObjectByName("resourcePop").caption = playerState.popCount + "/" + playerState.popLimit;
+	Engine.GetGUIObjectByName("population").tooltip = translate("Population (current / limit)") + "\n" +
+					sprintf(translate("Maximum population: %(popCap)s"), { "popCap": playerState.popMax });
 
 	g_IsTrainingBlocked = playerState.trainingBlocked;
 }
@@ -751,7 +760,7 @@ function updateResearchDisplay()
 	{
 		var button = Engine.GetGUIObjectByName("researchStartedButton[" + i + "]");
 		var size = button.size;
-		size.top = (4 + buttonSideLength) * i;
+		size.top = g_ResearchListTop + (4 + buttonSideLength) * i;
 		size.bottom = size.top + buttonSideLength;
 		button.size = size;
 	}
@@ -767,7 +776,7 @@ function updateResearchDisplay()
 		var button = Engine.GetGUIObjectByName("researchStartedButton[" + numButtons + "]");
 		button.hidden = false;
 		button.tooltip = getEntityNames(template);
-		button.onpress = (function(e) { return function() { selectAndMoveTo(e) } })(researchStarted[tech].researcher);
+		button.onpress = (function(e) { return function() { selectAndMoveTo(e); } })(researchStarted[tech].researcher);
 
 		var icon = "stretched:session/portraits/" + template.icon;
 		Engine.GetGUIObjectByName("researchStartedIcon[" + numButtons + "]").sprite = icon;
@@ -787,16 +796,6 @@ function updateResearchDisplay()
 		Engine.GetGUIObjectByName("researchStartedButton[" + i + "]").hidden = true;
 }
 
-function updateTimeElapsedCounter()
-{
-	var simState = GetSimState();
-	var timeElapsedCounter = Engine.GetGUIObjectByName("timeElapsedCounter");
-	if (g_CurrentSpeed != 1.0)
-		timeElapsedCounter.caption = sprintf(translate("%(time)s (%(speed)sx)"), { time: timeToString(simState.timeElapsed), speed: Engine.FormatDecimalNumberIntoString(g_CurrentSpeed) });
-	else
-		timeElapsedCounter.caption = timeToString(simState.timeElapsed);
-}
-
 // Toggles the display of status bars for all of the player's entities.
 function recalculateStatusBarDisplay()
 {
@@ -916,7 +915,8 @@ function reportGame(extendedSimState)
 		"Cavalry",
 		"Champion",
 		"Hero",
-		"Ship"
+		"Ship",
+		"Trader"
 	];
 	var unitsCountersTypes = [
 		"unitsTrained",
@@ -991,12 +991,13 @@ function reportGame(extendedSimState)
 	// Tribute
 	playerStatistics.tributesSent = "";
 	playerStatistics.tributesReceived = "";
-	// Total	
+	// Total
 	playerStatistics.economyScore = "";
 	playerStatistics.militaryScore = "";
 	playerStatistics.totalScore = "";
 	// Various
 	playerStatistics.treasuresCollected = "";
+	playerStatistics.lootCollected = "";
 	playerStatistics.feminisation = "";
 	playerStatistics.percentMapExplored = "";
 	var mapName = Engine.GetMapSettings().Name;
@@ -1006,8 +1007,11 @@ function reportGame(extendedSimState)
 	var teamsLocked = true;
 
 	// Serialize the statistics for each player into a comma-separated list.
-	for each (var player in extendedSimState.players)
+	// Ignore gaia
+	for (let i = 1; i < extendedSimState.players.length; ++i)
 	{
+		let player = extendedSimState.players[i];
+
 		playerStates += player.state + ",";
 		playerCivs += player.civ + ",";
 		teams += player.team + ",";
@@ -1037,6 +1041,7 @@ function reportGame(extendedSimState)
 		playerStatistics.tributesReceived += player.statistics.tributesReceived + ",";
 		playerStatistics.percentMapExplored += player.statistics.percentMapExplored + ",";
 		playerStatistics.treasuresCollected += player.statistics.treasuresCollected + ",";
+		playerStatistics.lootCollected += player.statistics.lootCollected + ",";
 	}
 
 	// Send the report with serialized data
@@ -1048,6 +1053,8 @@ function reportGame(extendedSimState)
 	reportObject.civs = playerCivs;
 	reportObject.teams = teams;
 	reportObject.teamsLocked = String(teamsLocked);
+	reportObject.ceasefireActive = String(extendedSimState.ceasefireActive);
+	reportObject.ceasefireTimeRemaining = String(extendedSimState.ceasefireTimeRemaining);
 	reportObject.mapName = mapName;
 	reportObject.economyScore = playerStatistics.economyScore;
 	reportObject.militaryScore = playerStatistics.militaryScore;
@@ -1076,8 +1083,8 @@ function reportGame(extendedSimState)
 	reportObject.tributesReceived = playerStatistics.tributesReceived;
 	reportObject.percentMapExplored = playerStatistics.percentMapExplored;
 	reportObject.treasuresCollected = playerStatistics.treasuresCollected;
+	reportObject.lootCollected = playerStatistics.lootCollected;
 	reportObject.tradeIncome = playerStatistics.tradeIncome;
 
 	Engine.SendGameReport(reportObject);
 }
-
